@@ -260,6 +260,9 @@ struct client
   struct array str_buf;
 
   generation_type generation;
+
+  struct result_object *object;
+  int noreply;
 };
 
 
@@ -351,6 +354,9 @@ client_init()
 
   c->generation = 1;            /* Different from initial command state.  */
 
+  c->object = NULL;
+  c->noreply = 0;
+
   return c;
 }
 
@@ -820,7 +826,7 @@ store_result(struct command_state *state, int res)
 {
   int index = get_index(state);
   next_index(state);
-  state->object->store(state->object->arg, (void *) res, index, NULL);
+  state->object->store(state->object->arg, (void *) (long) res, index, NULL);
 }
 
 
@@ -1322,7 +1328,7 @@ state_prepare(struct command_state *state)
 
       while (count > 0)
         {
-          iov->iov_base = (void *) (buf + (int) (iov->iov_base));
+          iov->iov_base = (void *) (buf + (long) (iov->iov_base));
           iov += step;
           count -= step;
         }
@@ -1569,25 +1575,26 @@ push_index(struct command_state *state, int index)
 static
 struct command_state *
 init_state(struct command_state *state, int index, size_t request_size,
-           size_t str_size, parse_reply_func parse_reply,
-           struct result_object *o, int *noreply)
+           size_t str_size, parse_reply_func parse_reply)
 {
-  state->object = o;
-
-  if (noreply && *noreply)
-    {
-      if (state->client->nowait || state->noreply)
-        parse_reply = NULL;
-      *noreply = state->noreply;
-      state->last_cmd_noreply = state->noreply;
-    }
-  else
-    {
-      state->last_cmd_noreply = 0;
-    }
-
   if (! is_active(state))
-    command_state_reset(state, (str_size > 0 ? request_size : 0), parse_reply);
+    {
+      if (state->client->noreply)
+        {
+          if (state->client->nowait || state->noreply)
+            parse_reply = NULL;
+
+          state->last_cmd_noreply = state->noreply;
+        }
+      else
+        {
+          state->last_cmd_noreply = 0;
+        }
+
+      state->object = state->client->object;
+      command_state_reset(state, (str_size > 0 ? request_size : 0),
+                          parse_reply);
+    }
 
   if (array_extend(state->iov_buf, struct iovec,
                    request_size, ARRAY_EXTEND_EXACT) == -1)
@@ -1610,9 +1617,9 @@ init_state(struct command_state *state, int index, size_t request_size,
       return NULL;
     }
 
-  if (parse_reply)
+  if (state->parse_reply)
     ++state->reply_count;
-  else if (! *noreply)
+  else if (! state->last_cmd_noreply)
     ++state->nowait_count;
 
   return state;
@@ -1623,7 +1630,7 @@ static
 struct command_state *
 get_state(struct client *c, int index, const char *key, size_t key_len,
           size_t request_size, size_t str_size,
-          parse_reply_func parse_reply, struct result_object *o, int *noreply)
+          parse_reply_func parse_reply)
 {
   struct server *s;
   int server_index, fd;
@@ -1639,18 +1646,31 @@ get_state(struct client *c, int index, const char *key, size_t key_len,
     return NULL;
 
   return init_state(&s->cmd_state, index, request_size, str_size,
-                    parse_reply, o, noreply);
+                    parse_reply);
+}
+
+
+static inline
+const char *
+get_noreply(struct command_state *state)
+{
+  if (state->noreply && state->client->noreply)
+    return " " NOREPLY;
+  else
+    return "";
 }
 
 
 inline
 void
-client_reset(struct client *c)
+client_reset(struct client *c, struct result_object *o, int noreply)
 {
   array_clear(c->index_list);
   array_clear(c->str_buf);
 
   ++c->generation;
+  c->object = o;
+  c->noreply = noreply;
 }
 
 
@@ -1661,8 +1681,7 @@ int
 client_prepare_set(struct client *c, enum set_cmd_e cmd, int key_index,
                    const char *key, size_t key_len,
                    flags_type flags, exptime_type exptime,
-                   const void *value, value_size_type value_size,
-                   struct result_object *o, int noreply)
+                   const void *value, value_size_type value_size)
 {
   static const size_t request_size = 6;
   static const size_t str_size =
@@ -1672,7 +1691,7 @@ client_prepare_set(struct client *c, enum set_cmd_e cmd, int key_index,
   struct command_state *state;
 
   state = get_state(c, key_index, key, key_len, request_size, str_size,
-                    parse_set_reply, o, &noreply);
+                    parse_set_reply);
   if (! state)
     return MEMCACHED_FAILURE;
 
@@ -1711,8 +1730,8 @@ client_prepare_set(struct client *c, enum set_cmd_e cmd, int key_index,
     char *buf = array_end(c->str_buf, char);
     size_t str_size =
       sprintf(buf, " " FMT_FLAGS " " FMT_EXPTIME " " FMT_VALUE_SIZE "%s\r\n",
-              flags, exptime, value_size, (noreply ? " " NOREPLY : ""));
-    iov_push(state, (void *) array_size(c->str_buf), str_size);
+              flags, exptime, value_size, get_noreply(state));
+    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -1727,8 +1746,7 @@ int
 client_prepare_cas(struct client *c, int key_index,
                    const char *key, size_t key_len,
                    cas_type cas, flags_type flags, exptime_type exptime,
-                   const void *value, value_size_type value_size,
-                   struct result_object *o, int noreply)
+                   const void *value, value_size_type value_size)
 {
   static const size_t request_size = 6;
   static const size_t str_size =
@@ -1738,7 +1756,7 @@ client_prepare_cas(struct client *c, int key_index,
   struct command_state *state;
 
   state = get_state(c, key_index, key, key_len, request_size, str_size,
-                    parse_set_reply, o, &noreply);
+                    parse_set_reply);
   if (! state)
     return MEMCACHED_FAILURE;
 
@@ -1753,8 +1771,8 @@ client_prepare_cas(struct client *c, int key_index,
     size_t str_size =
       sprintf(buf, " " FMT_FLAGS " " FMT_EXPTIME " " FMT_VALUE_SIZE
               " " FMT_CAS "%s\r\n", flags, exptime, value_size, cas,
-              (noreply ? " " NOREPLY : ""));
-    iov_push(state, (void *) array_size(c->str_buf), str_size);
+              get_noreply(state));
+    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -1767,14 +1785,14 @@ client_prepare_cas(struct client *c, int key_index,
 
 int
 client_prepare_get(struct client *c, enum get_cmd_e cmd, int key_index,
-                   const char *key, size_t key_len, struct result_object *o)
+                   const char *key, size_t key_len)
 {
   static const size_t request_size = 4;
 
   struct command_state *state;
 
   state = get_state(c, key_index, key, key_len, request_size, 0,
-                    parse_get_reply, o, NULL);
+                    parse_get_reply);
   if (! state)
     return MEMCACHED_FAILURE;
 
@@ -1814,8 +1832,7 @@ client_prepare_get(struct client *c, enum get_cmd_e cmd, int key_index,
 
 int
 client_prepare_incr(struct client *c, enum arith_cmd_e cmd, int key_index,
-                    const char *key, size_t key_len,
-                    arith_type arg, struct result_object *o, int noreply)
+                    const char *key, size_t key_len, arith_type arg)
 {
   static const size_t request_size = 4;
   static const size_t str_size = sizeof(" " ARITH_STUB " " NOREPLY "\r\n");
@@ -1823,7 +1840,7 @@ client_prepare_incr(struct client *c, enum arith_cmd_e cmd, int key_index,
   struct command_state *state;
 
   state = get_state(c, key_index, key, key_len, request_size, str_size,
-                    parse_arith_reply, o, &noreply);
+                    parse_arith_reply);
   if (! state)
     return MEMCACHED_FAILURE;
 
@@ -1845,8 +1862,8 @@ client_prepare_incr(struct client *c, enum arith_cmd_e cmd, int key_index,
   {
     char *buf = array_end(c->str_buf, char);
     size_t str_size =
-      sprintf(buf, " " FMT_ARITH "%s\r\n", arg, (noreply ? " " NOREPLY : ""));
-    iov_push(state, (void *) array_size(c->str_buf), str_size);
+      sprintf(buf, " " FMT_ARITH "%s\r\n", arg, get_noreply(state));
+    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -1856,8 +1873,7 @@ client_prepare_incr(struct client *c, enum arith_cmd_e cmd, int key_index,
 
 int
 client_prepare_delete(struct client *c, int key_index,
-                      const char *key, size_t key_len,
-                      delay_type delay, struct result_object *o, int noreply)
+                      const char *key, size_t key_len, delay_type delay)
 {
   static const size_t request_size = 4;
   static const size_t str_size = sizeof(" " DELAY_STUB " " NOREPLY "\r\n");
@@ -1865,7 +1881,7 @@ client_prepare_delete(struct client *c, int key_index,
   struct command_state *state;
 
   state = get_state(c, key_index, key, key_len, request_size, str_size,
-                    parse_delete_reply, o, &noreply);
+                    parse_delete_reply);
   if (! state)
     return MEMCACHED_FAILURE;
 
@@ -1878,9 +1894,8 @@ client_prepare_delete(struct client *c, int key_index,
   {
     char *buf = array_end(c->str_buf, char);
     size_t str_size =
-      sprintf(buf, " " FMT_DELAY "%s\r\n", delay,
-              (noreply ? " " NOREPLY : ""));
-    iov_push(state, (void *) array_size(c->str_buf), str_size);
+      sprintf(buf, " " FMT_DELAY "%s\r\n", delay, get_noreply(state));
+    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -1900,7 +1915,7 @@ client_flush_all(struct client *c, delay_type delay,
   double ddelay = delay, delay_step = 0.0;
   int i;
 
-  client_reset(c);
+  client_reset(c, o, noreply);
 
   if (array_size(c->servers) > 1)
     delay_step = ddelay / (array_size(c->servers) - 1);
@@ -1918,7 +1933,7 @@ client_flush_all(struct client *c, delay_type delay,
         continue;
 
       state = init_state(&s->cmd_state, i, request_size, str_size,
-                         parse_ok_reply, o, &noreply);
+                         parse_ok_reply);
       if (! state)
         continue;
 
@@ -1926,8 +1941,8 @@ client_flush_all(struct client *c, delay_type delay,
         char *buf = array_end(c->str_buf, char);
         size_t str_size =
           sprintf(buf, "flush_all " FMT_DELAY "%s\r\n",
-                  (delay_type) (ddelay + 0.5), (noreply ? " " NOREPLY : ""));
-        iov_push(state, (void *) array_size(c->str_buf), str_size);
+                  (delay_type) (ddelay + 0.5), get_noreply(state));
+        iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
         array_append(c->str_buf, str_size);
       }
     }
@@ -1944,7 +1959,7 @@ client_nowait_push(struct client *c)
   if (! c->nowait)
     return MEMCACHED_SUCCESS;
 
-  client_reset(c);
+  client_reset(c, NULL, 0);
 
   for (array_each(c->servers, struct server, s))
     {
@@ -1981,7 +1996,7 @@ client_server_versions(struct client *c, struct result_object *o)
   struct server *s;
   int i;
 
-  client_reset(c);
+  client_reset(c, o, 0);
 
   for (i = 0, array_each(c->servers, struct server, s), ++i)
     {
@@ -1993,7 +2008,7 @@ client_server_versions(struct client *c, struct result_object *o)
         continue;
 
       state = init_state(&s->cmd_state, i, request_size, 0,
-                         parse_version_reply, o, NULL);
+                         parse_version_reply);
       if (! state)
         continue;
 
@@ -2020,7 +2035,7 @@ client_noreply_push(struct client *c)
   struct server *s;
   int i;
 
-  client_reset(c);
+  client_reset(c, NULL, 0);
 
   for (i = 0, array_each(c->servers, struct server, s), ++i)
     {
@@ -2034,8 +2049,7 @@ client_noreply_push(struct client *c)
       if (fd == -1)
         continue;
 
-      state = init_state(state, i, request_size, 0,
-                         parse_nowait_reply, NULL, NULL);
+      state = init_state(state, i, request_size, 0, parse_nowait_reply);
       if (! state)
         continue;
 
