@@ -139,40 +139,36 @@ client_set_nowait(struct client *c, int enable);
 
 extern
 void
-client_reset(struct client *c);
+client_reset(struct client *c, struct result_object *o, int noreply);
 
 extern
 int
 client_prepare_set(struct client *c, enum set_cmd_e cmd, int key_index,
                    const char *key, size_t key_len,
                    flags_type flags, exptime_type exptime,
-                   const void *value, value_size_type value_size,
-                   struct result_object *o, int noreply);
+                   const void *value, value_size_type value_size);
 
 extern
 int
 client_prepare_cas(struct client *c, int key_index,
                    const char *key, size_t key_len,
                    cas_type cas, flags_type flags, exptime_type exptime,
-                   const void *value, value_size_type value_size,
-                   struct result_object *o, int noreply);
+                   const void *value, value_size_type value_size);
 
 extern
 int
 client_prepare_get(struct client *c, enum get_cmd_e cmd, int key_index,
-                   const char *key, size_t key_len, struct result_object *o);
+                   const char *key, size_t key_len);
 
 extern
 int
 client_prepare_incr(struct client *c, enum arith_cmd_e cmd, int key_index,
-                    const char *key, size_t key_len,
-                    arith_type arg, struct result_object *o, int noreply);
+                    const char *key, size_t key_len, arith_type arg);
 
 extern
 int
 client_prepare_delete(struct client *c, int key_index,
-                      const char *key, size_t key_len,
-                      delay_type delay, struct result_object *o, int noreply);
+                      const char *key, size_t key_len, delay_type delay);
 
 extern
 int
